==> cv-react-library-0.1.16.tgz: package/src/components/native/BarChart/BarChart.js <==
import React from 'react';
import PropTypes from 'prop-types';
import ChartWrapper from '../ChartWrapper/ChartWrapper';

import getStyles from './BarChart.styles';

/**
 * The bar chart shows different data through the height of a bar
 * which is used in rectangular coordinate with at least 1 category axis.
 */
const BarChart = (props) => {
    const {
        animation,
        barData,
        contextStyles,
        gridLines,
        horizontal,
        horizontalGridLines,
        legendText,
        maxX,
        maxY,
        minX,
        minY,
        onPress,
        reverse,
        testID,
        title,
        tooltip,
        valueLabelFormat,
        verticalGridLines,
        xAxis,
        xAxisTitle,
        yAxis,
        yAxisTitle,
        zoom,
    } = props;

    const styles = getStyles(contextStyles);

    /** Build axis options */
    const axisOptions = {
        xAxis: {
            axisTick: {
                // Move the tick from between the bars to the center of each bar
                alignWithLabel: true,
            },
            show: xAxis,
            splitLine: {
                // Show vertical grid lines
                show: gridLines && verticalGridLines,
            },
            type: 'category',
            data: [],
        },
        yAxis: {
            show: yAxis,
            splitLine: {
                // Show horizontal grid lines
                show: gridLines && horizontalGridLines,
            },
            type: 'value',
        },
    };

    // Show the bars from the top
    if (reverse && !horizontal) {
        axisOptions.xAxis.position = 'top';
        axisOptions.yAxis.inverse = true;
    }

    // Show the bars from the right
    if (reverse && horizontal) {
        axisOptions.xAxis.position = 'right';
        axisOptions.xAxis.inverse = true;
        axisOptions.yAxis.inverse = true;
    }

    // Show the bars from the left
    if (!reverse && horizontal) {
        axisOptions.xAxis.inverse = true;
    }

    /** Build chart options */
    const chartOptions = {
        animation,
        backgroundColor: 'transparent',
        grid: {
            containLabel: true,
            show: true,
        },
        xAxis: axisOptions.xAxis,
        yAxis: axisOptions.yAxis,
    };

    // Set chart title
    if (title) {
        chartOptions.title = {
            text: title,
        };
    }

    if (tooltip) {
        chartOptions.tooltip = {
            confine: true,
        };
    }

    // Set chart legend
    if (legendText) { chartOptions.legend = { data: [ legendText ] }; }

    // Map series data prop to chart data structure
    const mappedChartData = barData.map((item) => {
        // Build bar labels array
        axisOptions.xAxis.data.push(item.axisLabel);

        // Begin minimum bar detail
        const barDetail = {
            itemStyle: {},
            name: item.axisLabel,
            value: item.value,
        };

        // Add value label details
        if (item.valueLabel) {
            barDetail.label = {
                formatter: (item.valueLabelFormat || valueLabelFormat).replace('{value}', '{c}'),
                position: item.valueLabelPosition || 'inside',
                show: true,
            };

            // TODO: add label styles
            // barDetail.label.formatter = [
            //     '{a|Style "a" is applied to this snippet}',
            //     '{b|Style "b" is applied to this snippet}This snippet use default style{x|use style "x"}',
            // ].join('\n');

            // TODO: label dimensions
            // barDetail.label.width = 50;
            // barDetail.label.height = 75;

            // TODO: position by percentage
            // barDetail.label.position = ['50%', '50%'];

            // TODO: remove (for testing dimensions)
            // barDetail.label.borderWidth = 5;
            // barDetail.label.borderColor = '#00f';
            // barDetail.label.backgroundColor = '#123234';
            // barDetail.label.rich = {
            //     a: {
            //         backgroundColor: '#fff',
            //         color: '#fff',
            //     },
            // };

            if (item.valueLabelColor) { barDetail.label.color = item.valueLabelColor; }
            if (item.valueLabelSize) { barDetail.label.fontSize = item.valueLabelSize; }
        }

        // Add bar color details
        if (item.borderColor) {
            barDetail.itemStyle.barBorderColor = item.borderColor;
        }
        if (item.borderWidth || item.borderWidth === 0) {
            barDetail.itemStyle.barBorderWidth = item.borderWidth;
        }
        if (item.color) {
            barDetail.itemStyle.color = item.color;
        }
        if (item.opacity || item.opacity === 0) {
            barDetail.itemStyle.opacity = item.opacity;
        }
        if (item.emphasisColor) {
            barDetail.itemStyle.emphasis = { color: item.emphasisColor };
        }

        return barDetail;
    });

    /** Modify axis options */
    if (horizontal) {
        chartOptions.xAxis = { ...axisOptions.yAxis };
        chartOptions.yAxis = { ...axisOptions.xAxis };
    }

    // Set axis titles after potential reversal
    if (xAxisTitle) {
        chartOptions.xAxis.name = xAxisTitle;
        chartOptions.xAxis.nameLocation = 'center';
        chartOptions.xAxis.nameGap = 25;
    }
    if (yAxisTitle) {
        chartOptions.yAxis.name = yAxisTitle;
        chartOptions.yAxis.nameLocation = 'center';
        chartOptions.yAxis.nameGap = 30;
    }

    // Set range parameters after potential reversal
    if (maxX || maxX === 0) { chartOptions.xAxis.max = maxX; }
    if (minX || minX === 0) { chartOptions.xAxis.min = minX; }
    if (maxX || minX) { chartOptions.xAxis.scale = true; }
    if (maxY || maxY === 0) { chartOptions.yAxis.max = maxY; }
    if (minY || minY === 0) { chartOptions.yAxis.min = minY; }
    if (maxY || minY) { chartOptions.yAxis.scale = true; }

    /** Set chart options */
    const chartData = {
        type: 'bar',
        data: mappedChartData,
    };
    if (legendText) { chartData.name = legendText; }
    if (zoom) {
        chartOptions.dataZoom = [
            {
                type: 'slider',
                orient: 'horizontal',
            },
            {
                type: 'inside',
                orient: 'horizontal',
            },
        ];
    }
    chartOptions.series = [ chartData ];

    console.log('CHART OPTIONS: ', chartOptions);
    console.log('CHART OPTIONS: ', JSON.stringify(chartOptions));

    // Generate props
    const chartProps = {
        contextStyles: {
            container: styles.container,
        },
        option: chartOptions,
    };
    if (onPress) { chartProps.onPress = onPress; }
    if (testID) { chartProps.testID = testID; }

    return (
        <ChartWrapper { ...chartProps } />
    );
};

BarChart.propTypes = {
    /** Enables/disables chart animations */
    animation: PropTypes.bool,

    /** The details for each bar */
    barData: PropTypes.arrayOf(PropTypes.shape({
        /** Bar axis line text */
        axisLabel: PropTypes.string.isRequired,

        /** Color of the bar border */
        borderColor: PropTypes.string,

        /** Width of the bar border */
        borderWidth: PropTypes.number,

        /** Color of the bar */
        color: PropTypes.string,

        /** Highlight color of the bar on press */
        emphasisColor: PropTypes.string,

        /** Opacity of the bar */
        opacity: PropTypes.number,

        /** Value of the bar */
        value: PropTypes.number.isRequired,

        /** Display label inside the bar with the bar value */
        valueLabel: PropTypes.bool,

        /** Format string for the value label */
        valueLabelFormat: PropTypes.string,

        /** Position of value label within its bar */
        valueLabelPosition: PropTypes.oneOf([
            'inside',
            'insideLeft',
            'insideRight',
            'insideTop',
            'insideBottom',
            'insideTopLeft',
            'insideBottomLeft',
            'insideTopRight',
            'insideBottomRight',
        ]),

        /** Color of the bar value label */
        valueLabelColor: PropTypes.string,

        /** Font size of the bar value label */
        valueLabelSize: PropTypes.number,
    })).isRequired,

    /** Styles for the component */
    contextStyles: PropTypes.shape({
        /** Styles for the container */
        container: PropTypes.object,
    }),

    /** Enable display of grid lines */
    gridLines: PropTypes.bool,

    /** Displays the bars horizontally from left to right */
    horizontal: PropTypes.bool,

    /** Enable display of horizontal grid lines */
    horizontalGridLines: PropTypes.bool,

    /** Displays a legend symbol with text for the represented bars */
    legendText: PropTypes.string,

    /** Maximum X axis value */
    maxX: PropTypes.number,

    /** Maximum Y axis value */
    maxY: PropTypes.number,

    /** Minimum X axis value */
    minX: PropTypes.number,

    /** Minimum Y axis value */
    minY: PropTypes.number,

    /**
     * Triggered on press of a bar
     * @param {Object} itemData - Data for the bar pressed
     */
    onPress: PropTypes.func,

    /** Reverse orientation of the chart */
    reverse: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Title text for the chart */
    title: PropTypes.string,

    /** Show related data in a tooltip after pressing a bar */
    tooltip: PropTypes.bool,

    /** Format string for the value label */
    valueLabelFormat: PropTypes.string,

    /** Enable display of vertical grid lines */
    verticalGridLines: PropTypes.bool,

    /** Show X axis line, ticks and labels */
    xAxis: PropTypes.bool,

    /** Text to display along the X axis */
    xAxisTitle: PropTypes.string,

    /** Show Y axis line, ticks and labels */
    yAxis: PropTypes.bool,

    /** Text to display along the Y axis */
    yAxisTitle: PropTypes.string,

    /** Enable zooming in/out chart ranges */
    zoom: PropTypes.bool,
};

BarChart.defaultProps = {
    animation: true,
    contextStyles: {},
    gridLines: true,
    horizontalGridLines: true,
    tooltip: true,
    valueLabelFormat: '{value}',
    verticalGridLines: true,
    xAxis: true,
    xAxisTitle: 'X',
    yAxis: true,
    yAxisTitle: 'Y',
    zoom: true,
};

export default BarChart;

==> cv-react-library-0.1.21.tgz: package/src/components/native/BarChart/BarChart.js <==
import React from 'react';
import PropTypes from 'prop-types';
import ChartWrapper from '../ChartWrapper/ChartWrapper';

import getStyles from './BarChart.styles';

/**
 * The bar chart shows different data through the height of a bar
 * which is used in rectangular coordinate with at least 1 category axis.
 */
const BarChart = (props) => {
    const {
        animation,
        contextStyles,
        gridLines,
        horizontal,
        horizontalGridLines,
        legendText,
        maxX,
        maxY,
        minX,
        minY,
        onPress,
        reverse,
        seriesData,
        stacked,
        testID,
        title,
        tooltip,
        valueLabelFormat,
        verticalGridLines,
        xAxis,
        xAxisTitle,
        yAxis,
        yAxisTitle,
        zoom,
    } = props;

    const styles = getStyles(contextStyles);

    /** Build axis options */
    const axisOptions = {
        xAxis: {
            axisTick: {
                // Move the tick from between the bars to the center of each bar
                alignWithLabel: true,
            },
            data: [],
            show: xAxis,
            splitLine: {
                // Show vertical grid lines
                show: gridLines && verticalGridLines,
            },
            type: 'category',
        },
        yAxis: {
            show: yAxis,
            splitLine: {
                // Show horizontal grid lines
                show: gridLines && horizontalGridLines,
            },
            type: 'value',
        },
    };

    // Show the bars from the top
    if (reverse && !horizontal) {
        axisOptions.xAxis.position = 'top';
        axisOptions.yAxis.inverse = true;
    }

    // Show the bars from the right
    if (reverse && horizontal) {
        axisOptions.xAxis.position = 'right';
        axisOptions.xAxis.inverse = true;
        axisOptions.yAxis.inverse = true;
    }

    // Show the bars from the left
    if (!reverse && horizontal) {
        axisOptions.xAxis.inverse = true;
    }

    /** Build chart options */
    const chartOptions = {
        animation,
        backgroundColor: 'transparent',
        grid: {
            containLabel: true,
            show: true,
        },
        xAxis: axisOptions.xAxis,
        yAxis: axisOptions.yAxis,
    };

    // Set chart title
    if (title) {
        chartOptions.title = {
            text: title,
        };
    }

    // Enable tooltip
    if (tooltip) {
        chartOptions.tooltip = {
            confine: true,
        };
    }

    // Set chart legend
    if (legendText) { chartOptions.legend = { data: legendText }; }

    // Map series data prop to chart data structure
    const mappedChartData = seriesData.map((barDataSeries) => (
        barDataSeries.map((item) => {
            // Build bar labels array
            if (!axisOptions.xAxis.data.find((label) => (label === item.axisLabel))) {
                axisOptions.xAxis.data.push(item.axisLabel);
            }

            // Begin minimum bar detail
            const barDetail = {
                itemStyle: {},
                name: item.axisLabel,
                value: item.value,
            };

            // Add value label details
            if (item.valueLabel) {
                barDetail.label = {
                    formatter: (item.valueLabelFormat || valueLabelFormat).replace('{value}', '{c}'),
                    position: item.valueLabelPosition || 'inside',
                    show: true,
                };

                // TODO: add label styles
                // barDetail.label.formatter = [
                //     '{a|Style "a" is applied to this snippet}',
                //     '{b|Style "b" is applied to this snippet}This snippet use default style{x|use style "x"}',
                // ].join('\n');

                // TODO: label dimensions
                // barDetail.label.width = 50;
                // barDetail.label.height = 75;

                // TODO: position by percentage
                // barDetail.label.position = ['50%', '50%'];

                // TODO: remove (for testing dimensions)
                // barDetail.label.borderWidth = 5;
                // barDetail.label.borderColor = '#00f';
                // barDetail.label.backgroundColor = '#123234';
                // barDetail.label.rich = {
                //     a: {
                //         backgroundColor: '#fff',
                //         color: '#fff',
                //     },
                // };

                if (item.valueLabelColor) { barDetail.label.color = item.valueLabelColor; }
                if (item.valueLabelSize) { barDetail.label.fontSize = item.valueLabelSize; }
            }

            // Add bar color details
            if (item.borderColor) {
                barDetail.itemStyle.barBorderColor = item.borderColor;
            }
            if (item.borderWidth || item.borderWidth === 0) {
                barDetail.itemStyle.barBorderWidth = item.borderWidth;
            }
            if (item.color) {
                barDetail.itemStyle.color = item.color;
            }
            if (item.opacity || item.opacity === 0) {
                barDetail.itemStyle.opacity = item.opacity;
            }
            if (item.emphasisColor) {
                barDetail.itemStyle.emphasis = { color: item.emphasisColor };
            }

            return barDetail;
        })
    ));

    /** Modify axis options */
    if (horizontal) {
        chartOptions.xAxis = { ...axisOptions.yAxis };
        chartOptions.yAxis = { ...axisOptions.xAxis };
    }

    // Set axis titles after potential reversal
    if (xAxisTitle) {
        chartOptions.xAxis.name = xAxisTitle;
        chartOptions.xAxis.nameLocation = 'center';
        chartOptions.xAxis.nameGap = 25;
    }
    if (yAxisTitle) {
        chartOptions.yAxis.name = yAxisTitle;
        chartOptions.yAxis.nameLocation = 'center';
        chartOptions.yAxis.nameGap = 30;
    }

    // Set range parameters after potential reversal
    if (maxX || maxX === 0) { chartOptions.xAxis.max = maxX; }
    if (minX || minX === 0) { chartOptions.xAxis.min = minX; }
    if (maxX || minX) { chartOptions.xAxis.scale = true; }
    if (maxY || maxY === 0) { chartOptions.yAxis.max = maxY; }
    if (minY || minY === 0) { chartOptions.yAxis.min = minY; }
    if (maxY || minY) { chartOptions.yAxis.scale = true; }

    /** Set chart options */
    const seriesArray = mappedChartData.map((series, idx) => {
        const seriesData = {
            type: 'bar',
            data: series,
            barGap: 0,
        };
        if (legendText) { seriesData.name = legendText[idx]; }
        if (stacked) { seriesData.stack = 'BarStack'; }
        return seriesData;
    });

    // Enable zoom
    if (zoom) {
        chartOptions.dataZoom = [
            {
                type: 'slider',
                orient: 'horizontal',
            },
            {
                type: 'inside',
                orient: 'horizontal',
            },
        ];
    }
    chartOptions.series = seriesArray;

    // Generate props
    const chartProps = {
        contextStyles: {
            container: styles.container,
        },
        option: chartOptions,
    };
    if (onPress) { chartProps.onPress = onPress; }
    if (testID) { chartProps.testID = testID; }

    return (
        <ChartWrapper { ...chartProps } />
    );
};

BarChart.propTypes = {
    /** Enables/disables chart animations */
    animation: PropTypes.bool,

    /** Styles for the component */
    contextStyles: PropTypes.shape({
        /** Styles for the container */
        container: PropTypes.object,
    }),

    /** Enable display of grid lines */
    gridLines: PropTypes.bool,

    /** Displays the bars horizontally from left to right */
    horizontal: PropTypes.bool,

    /** Enable display of horizontal grid lines */
    horizontalGridLines: PropTypes.bool,

    /** Displays a legend symbol with text for the represented bars */
    legendText: PropTypes.arrayOf(PropTypes.string),

    /** Maximum X axis value */
    maxX: PropTypes.number,

    /** Maximum Y axis value */
    maxY: PropTypes.number,

    /** Minimum X axis value */
    minX: PropTypes.number,

    /** Minimum Y axis value */
    minY: PropTypes.number,

    /**
     * Triggered on press of a bar
     * @param {Object} itemData - Data for the bar pressed
     */
    onPress: PropTypes.func,

    /** Reverse orientation of the chart */
    reverse: PropTypes.bool,

    /** The details for each bar */
    seriesData: PropTypes.arrayOf(PropTypes.arrayOf(PropTypes.shape({
        /** Bar axis line text */
        axisLabel: PropTypes.string.isRequired,

        /** Color of the bar border */
        borderColor: PropTypes.string,

        /** Width of the bar border */
        borderWidth: PropTypes.number,

        /** Color of the bar */
        color: PropTypes.string,

        /** Highlight color of the bar on press */
        emphasisColor: PropTypes.string,

        /** Opacity of the bar */
        opacity: PropTypes.number,

        /** Value of the bar */
        value: PropTypes.number.isRequired,

        /** Display label inside the bar with the bar value */
        valueLabel: PropTypes.bool,

        /** Format string for the value label */
        valueLabelFormat: PropTypes.string,

        /** Position of value label within its bar */
        valueLabelPosition: PropTypes.oneOf([
            'inside',
            'insideLeft',
            'insideRight',
            'insideTop',
            'insideBottom',
            'insideTopLeft',
            'insideBottomLeft',
            'insideTopRight',
            'insideBottomRight',
        ]),

        /** Color of the bar value label */
        valueLabelColor: PropTypes.string,

        /** Font size of the bar value label */
        valueLabelSize: PropTypes.number,
    }))).isRequired,

    /** Stack bars */
    stacked: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Title text for the chart */
    title: PropTypes.string,

    /** Show related data in a tooltip after pressing a bar */
    tooltip: PropTypes.bool,

    /**
     * Format string for the value label
     * @see https://echarts.apache.org/en/option.html#series-scatter.label.formatter for the available formatting options
     */
    valueLabelFormat: PropTypes.string,

    /** Enable display of vertical grid lines */
    verticalGridLines: PropTypes.bool,

    /** Show X axis line, ticks and labels */
    xAxis: PropTypes.bool,

    /** Text to display along the X axis */
    xAxisTitle: PropTypes.string,

    /** Show Y axis line, ticks and labels */
    yAxis: PropTypes.bool,

    /** Text to display along the Y axis */
    yAxisTitle: PropTypes.string,

    /** Enable zooming in/out chart ranges */
    zoom: PropTypes.bool,
};

BarChart.defaultProps = {
    animation: true,
    contextStyles: {},
    gridLines: true,
    horizontalGridLines: true,
    tooltip: true,
    valueLabelFormat: '{value}',
    verticalGridLines: true,
    xAxis: true,
    xAxisTitle: 'X',
    yAxis: true,
    yAxisTitle: 'Y',
    zoom: true,
};

export default BarChart;
